# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Snuglite3(KaitaiStruct):
    """:field destination_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field source_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field csp_header_priority: payload.csp_header_priority
    :field csp_header_source: payload.csp_header_source
    :field csp_header_destination: payload.csp_header_destination
    :field csp_header_destination_port: payload.csp_header_destination_port
    :field csp_header_source_port: payload.csp_header_source_port
    :field csp_header_reserved: payload.csp_header_reserved
    :field csp_header_flags: payload.csp_header_flags
    
    :field last_telecommand_number: payload.last_telecommand_number
    :field telecommand_counter: payload.telecommand_counter
    :field gpstime_itow: payload.gpstime_itow
    :field gpstime_ftow: payload.gpstime_ftow
    :field position_flag: payload.position_flag
    :field position_x: payload.position_x
    :field position_y: payload.position_y
    :field position_z: payload.position_z
    :field velocity_x: payload.velocity_x
    :field velocity_y: payload.velocity_y
    :field velocity_z: payload.velocity_z
    :field battery_mode: payload.battery_mode
    :field battery_voltage: payload.battery_voltage
    :field target_altitude_q1: payload.target_altitude_q1
    :field target_altitude_q2: payload.target_altitude_q2
    :field target_altitude_q3: payload.target_altitude_q3
    :field estimated_altitude_q1: payload.estimated_altitude_q1
    :field estimated_altitude_q2: payload.estimated_altitude_q2
    :field estimated_altitude_q3: payload.estimated_altitude_q3
    :field estimated_angular_rate_roll: payload.estimated_angular_rate_roll
    :field estimated_angular_rate_pitch: payload.estimated_angular_rate_pitch
    :field estimated_angular_rate_yaw: payload.estimated_angular_rate_yaw
    :field sun_measurement_x: payload.sun_measurement_x
    :field sun_measurement_y: payload.sun_measurement_y
    :field sun_measurement_z: payload.sun_measurement_z
    :field relnav_mode: payload.relnav_mode
    :field ocs_mode: payload.ocs_mode
    :field ocs_mode_sate: payload.ocs_mode_sate
    :field target_relative_distance: payload.target_relative_distance
    :field current_relative_vector_x: payload.current_relative_vector_x
    :field current_relative_vector_y: payload.current_relative_vector_y
    :field current_relative_vector_z: payload.current_relative_vector_z
    :field current_relative_velocity_x: payload.current_relative_velocity_x
    :field current_relative_velocity_y: payload.current_relative_velocity_y
    :field current_relative_velocity_z: payload.current_relative_velocity_z
    :field operational_mode: payload.operational_mode
    :field deploy_flag: payload.deploy_flag
    :field satellite_time: payload.satellite_time
    
    :field page_active_bitmask: payload.full_beacon_page_number.page_number.page_active_bitmask
    :field battery_current: payload.full_beacon_page_number.page_number.battery_current
    :field power_switch_status_gps_0: payload.full_beacon_page_number.page_number.power_switch_status_gps_0
    :field power_switch_status_gps_1: payload.full_beacon_page_number.page_number.power_switch_status_gps_1
    :field power_switch_status_gps_2: payload.full_beacon_page_number.page_number.power_switch_status_gps_2
    :field power_switch_status_deploy: payload.full_beacon_page_number.page_number.power_switch_status_deploy
    :field power_switch_status_sub_trx: payload.full_beacon_page_number.page_number.power_switch_status_sub_trx
    :field power_switch_status_main_trx: payload.full_beacon_page_number.page_number.power_switch_status_main_trx
    :field power_switch_current_1: payload.full_beacon_page_number.page_number.power_switch_current_1
    :field power_switch_current_2: payload.full_beacon_page_number.page_number.power_switch_current_2
    :field power_switch_current_3: payload.full_beacon_page_number.page_number.power_switch_current_3
    :field power_switch_current_4: payload.full_beacon_page_number.page_number.power_switch_current_4
    :field power_switch_current_5: payload.full_beacon_page_number.page_number.power_switch_current_5
    :field power_switch_current_6: payload.full_beacon_page_number.page_number.power_switch_current_6
    :field solar_panel_input_voltage_z: payload.full_beacon_page_number.page_number.solar_panel_input_voltage_z
    :field solar_panel_input_voltage_y: payload.full_beacon_page_number.page_number.solar_panel_input_voltage_y
    :field solar_panel_input_voltage_deployed_panel: payload.full_beacon_page_number.page_number.solar_panel_input_voltage_deployed_panel
    :field solar_panel_input_current_z: payload.full_beacon_page_number.page_number.solar_panel_input_current_z
    :field solar_panel_input_current_y: payload.full_beacon_page_number.page_number.solar_panel_input_current_y
    :field solar_panel_input_current_deployed_panel: payload.full_beacon_page_number.page_number.solar_panel_input_current_deployed_panel
    :field mode_1: payload.full_beacon_page_number.page_number.mode_1
    :field mode_2: payload.full_beacon_page_number.page_number.mode_2
    :field mode_entry_time: payload.full_beacon_page_number.page_number.mode_entry_time
    :field dgps_relative_vector_x: payload.full_beacon_page_number.page_number.dgps_relative_vector_x
    :field dgps_relative_vector_y: payload.full_beacon_page_number.page_number.dgps_relative_vector_y
    :field dgps_relative_vector_z: payload.full_beacon_page_number.page_number.dgps_relative_vector_z
    :field raf_relative_vector_x: payload.full_beacon_page_number.page_number.raf_relative_vector_x
    :field raf_relative_vector_y: payload.full_beacon_page_number.page_number.raf_relative_vector_y
    :field raf_relative_vector_z: payload.full_beacon_page_number.page_number.raf_relative_vector_z
    :field rtk_relative_vector_x: payload.full_beacon_page_number.page_number.rtk_relative_vector_x
    :field rtk_relative_vector_y: payload.full_beacon_page_number.page_number.rtk_relative_vector_y
    :field rtk_relative_vector_z: payload.full_beacon_page_number.page_number.rtk_relative_vector_z
    :field id2: payload.full_beacon_page_number.page_number.id2
    :field page_type: payload.full_beacon_page_number.page_number.page_type
    
    :field page_active_bitmask: payload.full_beacon_page_number.page_number.page_active_bitmask
    :field star_tracker_q1: payload.full_beacon_page_number.page_number.star_tracker_q1
    :field star_tracker_q2: payload.full_beacon_page_number.page_number.star_tracker_q2
    :field star_tracker_q3: payload.full_beacon_page_number.page_number.star_tracker_q3
    :field gyroscope_x: payload.full_beacon_page_number.page_number.gyroscope_x
    :field gyroscope_y: payload.full_beacon_page_number.page_number.gyroscope_y
    :field gyroscope_z: payload.full_beacon_page_number.page_number.gyroscope_z
    :field magnetometer_x: payload.full_beacon_page_number.page_number.magnetometer_x
    :field magnetometer_y: payload.full_beacon_page_number.page_number.magnetometer_y
    :field magnetometer_z: payload.full_beacon_page_number.page_number.magnetometer_z
    :field estimated_gyro_bias_roll: payload.full_beacon_page_number.page_number.estimated_gyro_bias_roll
    :field estimated_gyro_bias_pitch: payload.full_beacon_page_number.page_number.estimated_gyro_bias_pitch
    :field estimated_gyro_bias_yaw: payload.full_beacon_page_number.page_number.estimated_gyro_bias_yaw
    :field reaction_wheel_speed_x: payload.full_beacon_page_number.page_number.reaction_wheel_speed_x
    :field reaction_wheel_speed_y: payload.full_beacon_page_number.page_number.reaction_wheel_speed_y
    :field reaction_wheel_speed_z: payload.full_beacon_page_number.page_number.reaction_wheel_speed_z
    :field external_panel_plus_x: payload.full_beacon_page_number.page_number.external_panel_plus_x
    :field external_panel_plus_y: payload.full_beacon_page_number.page_number.external_panel_plus_y
    :field external_panel_minus_y: payload.full_beacon_page_number.page_number.external_panel_minus_y
    :field external_panel_plus_z: payload.full_beacon_page_number.page_number.external_panel_plus_z
    :field external_panel_minus_z: payload.full_beacon_page_number.page_number.external_panel_minus_z
    :field obc_main_1: payload.full_beacon_page_number.page_number.obc_main_1
    :field obc_main_2: payload.full_beacon_page_number.page_number.obc_main_2
    :field obc_sub_1: payload.full_beacon_page_number.page_number.obc_sub_1
    :field obc_sub_2: payload.full_beacon_page_number.page_number.obc_sub_2
    :field eps_board_1: payload.full_beacon_page_number.page_number.eps_board_1
    :field eps_board_2: payload.full_beacon_page_number.page_number.eps_board_2
    :field eps_board_3: payload.full_beacon_page_number.page_number.eps_board_3
    :field eps_board_4: payload.full_beacon_page_number.page_number.eps_board_4
    :field eps_battery_1: payload.full_beacon_page_number.page_number.eps_battery_1
    :field eps_battery_2: payload.full_beacon_page_number.page_number.eps_battery_2
    :field obc_gpio_1: payload.full_beacon_page_number.page_number.obc_gpio_1
    :field obc_gpio_2: payload.full_beacon_page_number.page_number.obc_gpio_2
    :field obc_pwm: payload.full_beacon_page_number.page_number.obc_pwm
    :field relnav_flag_1: payload.full_beacon_page_number.page_number.relnav_flag_1
    :field relnav_flag_2: payload.full_beacon_page_number.page_number.relnav_flag_2
    :field relnav_flag_3: payload.full_beacon_page_number.page_number.relnav_flag_3
    :field id2: payload.full_beacon_page_number.page_number.id2
    :field page_type: payload.full_beacon_page_number.page_number.page_type
    
    .. seealso::
       Source - https://gnss.snu.ac.kr/snuglite/251126/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Snuglite3.Ax25Frame(self._io, self, self._root)
        self.payload = Snuglite3.Payload(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Snuglite3.Ax25Header(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Snuglite3.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Snuglite3.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Snuglite3.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Snuglite3.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"DS0DH") or (self.callsign == u"DS0DH ") or (self.callsign == u"DS0DH0")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class Payload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_header_priority = self._io.read_bits_int_be(2)
            self.csp_header_source = self._io.read_bits_int_be(5)
            self.csp_header_destination = self._io.read_bits_int_be(5)
            self.csp_header_destination_port = self._io.read_bits_int_be(6)
            self.csp_header_source_port = self._io.read_bits_int_be(6)
            self.csp_header_reserved = self._io.read_bits_int_be(4)
            self.csp_header_flags = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.id1 = (self._io.read_bytes(7)).decode(u"UTF-8")
            if not self.id1 == u"SNUGL3>":
                raise kaitaistruct.ValidationNotEqualError(u"SNUGL3>", self.id1, self._io, u"/types/payload/seq/7")
            self.last_telecommand_number = self._io.read_u1()
            self.telecommand_counter = self._io.read_u1()
            self.time_year = self._io.read_u1()
            self.time_month = self._io.read_u1()
            self.time_day = self._io.read_u1()
            self.time_hour = self._io.read_u1()
            self.time_minute = self._io.read_u1()
            self.time_second = self._io.read_u1()
            self.gpstime_itow = self._io.read_s4be()
            self.gpstime_ftow = self._io.read_s4be()
            self.position_flag = self._io.read_u1()
            self.position_x = self._io.read_s4be()
            self.position_y = self._io.read_s4be()
            self.position_z = self._io.read_s4be()
            self.velocity_x = self._io.read_s4be()
            self.velocity_y = self._io.read_s4be()
            self.velocity_z = self._io.read_s4be()
            self.battery_mode = self._io.read_u1()
            self.battery_voltage = self._io.read_u2be()
            self.target_altitude_q1 = self._io.read_s2be()
            self.target_altitude_q2 = self._io.read_s2be()
            self.target_altitude_q3 = self._io.read_s2be()
            self.estimated_altitude_q1 = self._io.read_s2be()
            self.estimated_altitude_q2 = self._io.read_s2be()
            self.estimated_altitude_q3 = self._io.read_s2be()
            self.estimated_angular_rate_roll = self._io.read_f4be()
            self.estimated_angular_rate_pitch = self._io.read_f4be()
            self.estimated_angular_rate_yaw = self._io.read_f4be()
            self.sun_measurement_x = self._io.read_s2be()
            self.sun_measurement_y = self._io.read_s2be()
            self.sun_measurement_z = self._io.read_s2be()
            self.relnav_mode = self._io.read_u1()
            self.ocs_mode = self._io.read_u1()
            self.ocs_mode_sate = self._io.read_u1()
            self.target_relative_distance = self._io.read_s2be()
            self.current_relative_vector_x = self._io.read_f4be()
            self.current_relative_vector_y = self._io.read_f4be()
            self.current_relative_vector_z = self._io.read_f4be()
            self.current_relative_velocity_x = self._io.read_f4be()
            self.current_relative_velocity_y = self._io.read_f4be()
            self.current_relative_velocity_z = self._io.read_f4be()
            self.operational_mode = self._io.read_bits_int_le(3)
            self.skipping_rest_2 = self._io.read_bits_int_be(5)
            self._io.align_to_byte()
            self.deploy_flag = self._io.read_u1()
            self.full_beacon_page_number = Snuglite3.FullBeaconPageNumberT(self._io, self, self._root)

        @property
        def month(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_month'):
                return self._m_month

            self._m_month = (u"0" + str(self.time_month) if len(str(self.time_month)) == 1 else str(self.time_month))
            return getattr(self, '_m_month', None)

        @property
        def minute(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_minute'):
                return self._m_minute

            self._m_minute = (u"0" + str(self.time_minute) if len(str(self.time_minute)) == 1 else str(self.time_minute))
            return getattr(self, '_m_minute', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            self._m_satellite_time = u"20" + self.year + u"-" + self.month + u"-" + self.day + u"T" + self.hour + u":" + self.minute + u":" + self.second + u"Z"
            return getattr(self, '_m_satellite_time', None)

        @property
        def year(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_year'):
                return self._m_year

            self._m_year = (u"0" + str(self.time_year) if len(str(self.time_year)) == 1 else str(self.time_year))
            return getattr(self, '_m_year', None)

        @property
        def second(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_second'):
                return self._m_second

            self._m_second = (u"0" + str(self.time_second) if len(str(self.time_second)) == 1 else str(self.time_second))
            return getattr(self, '_m_second', None)

        @property
        def day(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_day'):
                return self._m_day

            self._m_day = (u"0" + str(self.time_day) if len(str(self.time_day)) == 1 else str(self.time_day))
            return getattr(self, '_m_day', None)

        @property
        def hour(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_hour'):
                return self._m_hour

            self._m_hour = (u"0" + str(self.time_hour) if len(str(self.time_hour)) == 1 else str(self.time_hour))
            return getattr(self, '_m_hour', None)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 31) >> 1)
            return getattr(self, '_m_ssid', None)


    class Page1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.page_active_bitmask = self._io.read_bits_int_le(4)
            self.skipping_rest_0 = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.battery_current = self._io.read_u2be()
            self.power_switch_status_gps_0 = self._io.read_bits_int_le(1) != 0
            self.power_switch_status_gps_1 = self._io.read_bits_int_le(1) != 0
            self.power_switch_status_gps_2 = self._io.read_bits_int_le(1) != 0
            self.power_switch_status_deploy = self._io.read_bits_int_le(1) != 0
            self.power_switch_status_sub_trx = self._io.read_bits_int_le(1) != 0
            self.power_switch_status_main_trx = self._io.read_bits_int_le(1) != 0
            self.skipping_rest_1 = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.power_switch_current_1 = self._io.read_u2be()
            self.power_switch_current_2 = self._io.read_u2be()
            self.power_switch_current_3 = self._io.read_u2be()
            self.power_switch_current_4 = self._io.read_u2be()
            self.power_switch_current_5 = self._io.read_u2be()
            self.power_switch_current_6 = self._io.read_u2be()
            self.solar_panel_input_voltage_z = self._io.read_u2be()
            self.solar_panel_input_voltage_y = self._io.read_u2be()
            self.solar_panel_input_voltage_deployed_panel = self._io.read_u2be()
            self.solar_panel_input_current_z = self._io.read_u2be()
            self.solar_panel_input_current_y = self._io.read_u2be()
            self.solar_panel_input_current_deployed_panel = self._io.read_u2be()
            self.mode_1 = self._io.read_u1()
            self.mode_2 = self._io.read_u1()
            self.mode_entry_time = self._io.read_u4be()
            self.dgps_relative_vector_x = self._io.read_f4be()
            self.dgps_relative_vector_y = self._io.read_f4be()
            self.dgps_relative_vector_z = self._io.read_f4be()
            self.raf_relative_vector_x = self._io.read_f4be()
            self.raf_relative_vector_y = self._io.read_f4be()
            self.raf_relative_vector_z = self._io.read_f4be()
            self.rtk_relative_vector_x = self._io.read_f4be()
            self.rtk_relative_vector_y = self._io.read_f4be()
            self.rtk_relative_vector_z = self._io.read_f4be()
            self.id2 = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not  ((self.id2 == u"<HANA") or (self.id2 == u"<DURI")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.id2, self._io, u"/types/page1/seq/34")
            self.skipping_last_byte = self._io.read_u1()

        @property
        def page_type(self):
            if hasattr(self, '_m_page_type'):
                return self._m_page_type

            self._m_page_type = (u"1" if 0 == 0 else u"1")
            return getattr(self, '_m_page_type', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Snuglite3.Callsign(_io__raw_callsign_ror, self, self._root)


    class Page2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.page_active_bitmask = self._io.read_bits_int_le(4)
            self.skipping_rest = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.star_tracker_q1 = self._io.read_s2be()
            self.star_tracker_q2 = self._io.read_s2be()
            self.star_tracker_q3 = self._io.read_s2be()
            self.gyroscope_x = self._io.read_f4be()
            self.gyroscope_y = self._io.read_f4be()
            self.gyroscope_z = self._io.read_f4be()
            self.magnetometer_x = self._io.read_f4be()
            self.magnetometer_y = self._io.read_f4be()
            self.magnetometer_z = self._io.read_f4be()
            self.estimated_gyro_bias_roll = self._io.read_f4be()
            self.estimated_gyro_bias_pitch = self._io.read_f4be()
            self.estimated_gyro_bias_yaw = self._io.read_f4be()
            self.reaction_wheel_speed_x = self._io.read_s2be()
            self.reaction_wheel_speed_y = self._io.read_s2be()
            self.reaction_wheel_speed_z = self._io.read_s2be()
            self.external_panel_plus_x = self._io.read_u1()
            self.external_panel_plus_y = self._io.read_u1()
            self.external_panel_minus_y = self._io.read_u1()
            self.external_panel_plus_z = self._io.read_u1()
            self.external_panel_minus_z = self._io.read_u1()
            self.obc_main_1 = self._io.read_u1()
            self.obc_main_2 = self._io.read_u1()
            self.obc_sub_1 = self._io.read_u1()
            self.obc_sub_2 = self._io.read_u1()
            self.eps_board_1 = self._io.read_u1()
            self.eps_board_2 = self._io.read_u1()
            self.eps_board_3 = self._io.read_u1()
            self.eps_board_4 = self._io.read_u1()
            self.eps_battery_1 = self._io.read_u1()
            self.eps_battery_2 = self._io.read_u1()
            self.obc_gpio_1 = self._io.read_u1()
            self.obc_gpio_2 = self._io.read_u1()
            self.obc_pwm = self._io.read_u1()
            self.relnav_flag_1 = self._io.read_u1()
            self.relnav_flag_2 = self._io.read_u1()
            self.relnav_flag_3 = self._io.read_u1()
            self.id2 = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not  ((self.id2 == u"<HANA") or (self.id2 == u"<DURI")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.id2, self._io, u"/types/page2/seq/38")
            self.skipping_last_byte = self._io.read_u1()

        @property
        def page_type(self):
            if hasattr(self, '_m_page_type'):
                return self._m_page_type

            self._m_page_type = (u"2" if 0 == 0 else u"2")
            return getattr(self, '_m_page_type', None)


    class FullBeaconPageNumberT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 0:
                self.page_number = Snuglite3.Page1(self._io, self, self._root)
            elif _on == 1:
                self.page_number = Snuglite3.Page2(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            self._m_check = self._io.read_bits_int_le(2)
            return getattr(self, '_m_check', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Aepex(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field ccsds_version: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.ccsds_version
    :field packet_type: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.packet_type
    :field secondary_header_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.secondary_header_flag
    :field is_stored_data: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.is_stored_data
    :field application_process_id: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.application_process_id
    :field grouping_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.grouping_flag
    :field sequence_count: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.sequence_count
    :field packet_length: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.packet_length
    :field time_stamp_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.time_stamp_seconds
    :field sub_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.sub_seconds
    :field padding: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.padding
    :field sw_cmd_recv_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_recv_count
    :field sw_cmd_fmt_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_fmt_count
    :field sw_cmd_rjct_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_rjct_count
    :field sw_cmd_succ_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_succ_count
    :field padding1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.padding1
    :field padding2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.padding2
    :field sw_cmd_fail_code: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_fail_code
    :field sw_cmd_xsum_state: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_xsum_state
    :field reusable_spare_1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_1
    :field reusable_spare_2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_2
    :field reusable_spare_3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_3
    :field reusable_spare_4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_4
    :field reusable_spare_5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_5
    :field sw_cmd_arm_state_uhf: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_arm_state_uhf
    :field sw_cmd_arm_state_seq: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_arm_state_seq
    :field sw_cmd_arm_state_dbg: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_cmd_arm_state_dbg
    :field reusable_spare_6: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_6
    :field sw_eps_pwr_state_inst4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_inst4
    :field sw_eps_pwr_state_inst3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_inst3
    :field sw_eps_pwr_state_inst2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_inst2
    :field sw_eps_pwr_state_inst1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_inst1
    :field sw_eps_pwr_state_sband: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_sband
    :field sw_eps_pwr_state_uhf: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_uhf
    :field sw_eps_pwr_state_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_eps_pwr_state_adcs
    :field sw_time_since_rx: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_time_since_rx
    :field reusable_spare_7: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_7
    :field reusable_spare_8: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_8
    :field reusable_spare_9: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_9
    :field reusable_spare_10: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_10
    :field reusable_spare_11: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_11
    :field reusable_spare_12: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_12
    :field reusable_spare_13: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_13
    :field sw_bat_alive_state_battery0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_bat_alive_state_battery0
    :field sw_mode_clt_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_mode_clt_count
    :field sw_mode_system_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_mode_system_mode
    :field reusable_spare_14: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_14
    :field sw_sband_pa_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_sband_pa_temp
    :field sw_sband_pa_curr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_sband_pa_curr
    :field reusable_spare_15: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_15
    :field sw_uhf_alive: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_uhf_alive
    :field sw_uhf_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_uhf_temp
    :field reusable_spare_16: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_16
    :field sw_seq_state_auto: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_state_auto
    :field sw_seq_state_op1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_state_op1
    :field sw_seq_state_op2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_state_op2
    :field sw_seq_state_op3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_state_op3
    :field sw_seq_stop_code_auto: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_stop_code_auto
    :field sw_seq_stop_code_op1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_stop_code_op1
    :field sw_seq_stop_code_op2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_stop_code_op2
    :field sw_seq_stop_code_op3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_stop_code_op3
    :field sw_seq_exec_buf_auto: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_exec_buf_auto
    :field sw_seq_exec_buf_op1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_exec_buf_op1
    :field sw_seq_exec_buf_op2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_exec_buf_op2
    :field sw_seq_exec_buf_op3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_seq_exec_buf_op3
    :field sw_store_partition_write_misc: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_write_misc
    :field sw_store_partition_read_misc: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_read_misc
    :field sw_store_partition_write_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_write_adcs
    :field sw_store_partition_read_adcs: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_read_adcs
    :field sw_store_partition_write_beacon: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_write_beacon
    :field sw_store_partition_read_beacon: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_read_beacon
    :field sw_store_partition_write_log: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_write_log
    :field sw_store_partition_read_log: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_read_log
    :field sw_store_partition_write_sci: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_write_sci
    :field sw_store_partition_read_sci: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_store_partition_read_sci
    :field sw_fp_resp_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_fp_resp_count
    :field sw_ana_bus_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_bus_v
    :field sw_ana_3p3_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_3p3_v
    :field sw_ana_2p5_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_2p5_v
    :field sw_ana_1p8_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_1p8_v
    :field sw_ana_1p0_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_1p0_v
    :field sw_ana_3p3_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_3p3_i
    :field sw_ana_1p8_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_1p8_i
    :field sw_ana_1p0_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_1p0_i
    :field sw_ana_cdh_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_cdh_temp
    :field sw_ana_cdh_3p3_ref: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_cdh_3p3_ref
    :field sw_ana_sa1_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sa1_v
    :field sw_ana_sa1_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sa1_i
    :field sw_ana_sa2_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sa2_v
    :field sw_ana_sa2_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sa2_i
    :field sw_ana_bat1_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_bat1_v
    :field sw_ana_eps_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_eps_temp
    :field sw_ana_eps_3p3_ref: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_eps_3p3_ref
    :field sw_ana_eps_bus_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_eps_bus_v
    :field sw_ana_eps_bus_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_eps_bus_i
    :field sw_ana_xact_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_xact_v
    :field sw_ana_xact_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_xact_i
    :field sw_ana_uhf_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_uhf_v
    :field sw_ana_uhf_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_uhf_i
    :field sw_ana_sband_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sband_v
    :field sw_ana_sband_i: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_sband_i
    :field sw_ana_axis1_volt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis1_volt
    :field sw_ana_axis1_curr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis1_curr
    :field sw_ana_axis2_volt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis2_volt
    :field sw_ana_axis2_curr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis2_curr
    :field sw_ana_axis3_volt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis3_volt
    :field sw_ana_axis3_curr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_axis3_curr
    :field sw_ana_afire_volt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_afire_volt
    :field sw_ana_afire_curr: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_afire_curr
    :field sw_ana_ifb_therm1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_ana_ifb_therm1
    :field sw_adcs_alive: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_alive
    :field sw_adcs_eclipse: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_eclipse
    :field sw_adcs_att_valid: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_att_valid
    :field sw_adcs_ref_valid: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_ref_valid
    :field sw_adcs_time_valid: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_time_valid
    :field sw_adcs_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_mode
    :field sw_adcs_recommend_sun_point: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_recommend_sun_point
    :field sw_adcs_sun_point_state: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_sun_point_state
    :field reusable_spare_17: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_17
    :field sw_adcs_analogs_voltage_2p5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_voltage_2p5
    :field sw_adcs_analogs_voltage_1p8: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_voltage_1p8
    :field sw_adcs_analogs_voltage_1p0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_voltage_1p0
    :field sw_adcs_analogs_det_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_det_temp
    :field sw_adcs_analogs_motor1_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_motor1_temp
    :field sw_adcs_analogs_motor2_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_motor2_temp
    :field sw_adcs_analogs_motor3_temp: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_motor3_temp
    :field sw_adcs_analogs_digital_bus_v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_analogs_digital_bus_v
    :field sw_adcs_cmd_acpt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_cmd_acpt
    :field sw_adcs_cmd_fail: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_cmd_fail
    :field sw_adcs_sun_vec1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_sun_vec1
    :field sw_adcs_sun_vec2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_sun_vec2
    :field sw_adcs_sun_vec3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_sun_vec3
    :field sw_adcs_wheel_sp1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_wheel_sp1
    :field sw_adcs_wheel_sp2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_wheel_sp2
    :field sw_adcs_wheel_sp3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_wheel_sp3
    :field sw_adcs_body_rt1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_rt1
    :field sw_adcs_body_rt2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_rt2
    :field sw_adcs_body_rt3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_rt3
    :field sw_adcs_body_quat1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_quat1
    :field sw_adcs_body_quat2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_quat2
    :field sw_adcs_body_quat3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_quat3
    :field sw_adcs_body_quat4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_adcs_body_quat4
    :field des_met_time_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..des_met_time_seconds
    :field sw_im_id: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..sw_im_id
    :field payload_alive_axis1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..payload_alive_axis1
    :field payload_alive_axis2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..payload_alive_axis2
    :field payload_alive_axis3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..payload_alive_axis3
    :field payload_alive_afire: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..payload_alive_afire
    :field reusable_spare_18: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_18
    :field reusable_spare_18: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_19
    :field reusable_spare_18: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..reusable_spare_20
    :field reusable_spare_18: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field..checksum
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Aepex.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Aepex.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Aepex.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Aepex.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Aepex.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Aepex.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Aepex.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Aepex.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Aepex.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Aepex.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Aepex.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Aepex.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Aepex.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"APX   ") or (self.callsign == u"LASP  ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Aepex.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DataSectionT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if self._parent.packet_primary_header.secondary_header_flag:
                self._raw_secondary_header = self._io.read_bytes(6)
                _io__raw_secondary_header = KaitaiStream(BytesIO(self._raw_secondary_header))
                self.secondary_header = Aepex.SecondaryHeaderT(_io__raw_secondary_header, self, self._root)

            _on = self._parent.packet_primary_header.application_process_id
            if _on == 1:
                self.user_data_field = Aepex.AepexSwStat(self._io, self, self._root)


    class SwStatT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sw_cmd_recv_count = self._io.read_u2be()
            self.sw_cmd_fmt_count = self._io.read_u2be()
            self.sw_cmd_rjct_count = self._io.read_u2be()
            self.sw_cmd_succ_count = self._io.read_u2be()
            self.padding1 = self._io.read_u2be()
            self.padding2 = self._io.read_u2be()
            self.sw_cmd_fail_code = self._io.read_u1()
            self.sw_cmd_xsum_state = self._io.read_u1()
            self.reusable_spare_1 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_2 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_3 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_4 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_5 = self._io.read_bits_int_be(1) != 0
            self.sw_cmd_arm_state_uhf = self._io.read_bits_int_be(1) != 0
            self.sw_cmd_arm_state_seq = self._io.read_bits_int_be(1) != 0
            self.sw_cmd_arm_state_dbg = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_6 = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_inst4 = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_inst3 = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_inst2 = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_inst1 = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_sband = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_uhf = self._io.read_bits_int_be(1) != 0
            self.sw_eps_pwr_state_adcs = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.sw_time_since_rx = self._io.read_u4be()
            self.reusable_spare_7 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_8 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_9 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_10 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_11 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_12 = self._io.read_bits_int_be(1) != 0
            self.reusable_spare_13 = self._io.read_bits_int_be(1) != 0
            self.sw_bat_alive_state_battery0 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.sw_mode_clt_count = self._io.read_u1()
            self.sw_mode_system_mode = self._io.read_u1()
            self.reusable_spare_14 = self._io.read_u1()
            self.sw_sband_pa_temp = self._io.read_u2be()
            self.sw_sband_pa_curr = self._io.read_u2be()
            self.reusable_spare_15 = self._io.read_u1()
            self.sw_uhf_alive = self._io.read_u1()
            self.sw_uhf_temp = self._io.read_s1()
            self.reusable_spare_16 = self._io.read_u1()
            self.sw_seq_state_auto = self._io.read_u1()
            self.sw_seq_state_op1 = self._io.read_u1()
            self.sw_seq_state_op2 = self._io.read_u1()
            self.sw_seq_state_op3 = self._io.read_u1()
            self.sw_seq_stop_code_auto = self._io.read_u1()
            self.sw_seq_stop_code_op1 = self._io.read_u1()
            self.sw_seq_stop_code_op2 = self._io.read_u1()
            self.sw_seq_stop_code_op3 = self._io.read_u1()
            self.sw_seq_exec_buf_auto = self._io.read_u2be()
            self.sw_seq_exec_buf_op1 = self._io.read_u2be()
            self.sw_seq_exec_buf_op2 = self._io.read_u2be()
            self.sw_seq_exec_buf_op3 = self._io.read_u2be()
            self.sw_store_partition_write_misc = self._io.read_u4be()
            self.sw_store_partition_read_misc = self._io.read_u4be()
            self.sw_store_partition_write_adcs = self._io.read_u4be()
            self.sw_store_partition_read_adcs = self._io.read_u4be()
            self.sw_store_partition_write_beacon = self._io.read_u4be()
            self.sw_store_partition_read_beacon = self._io.read_u4be()
            self.sw_store_partition_write_log = self._io.read_u4be()
            self.sw_store_partition_read_log = self._io.read_u4be()
            self.sw_store_partition_write_sci = self._io.read_u4be()
            self.sw_store_partition_read_sci = self._io.read_u4be()
            self.sw_fp_resp_count = self._io.read_u2be()
            self.sw_ana_bus_v = self._io.read_u2be()
            self.sw_ana_3p3_v = self._io.read_u2be()
            self.sw_ana_2p5_v = self._io.read_u2be()
            self.sw_ana_1p8_v = self._io.read_u2be()
            self.sw_ana_1p0_v = self._io.read_u2be()
            self.sw_ana_3p3_i = self._io.read_u2be()
            self.sw_ana_1p8_i = self._io.read_u2be()
            self.sw_ana_1p0_i = self._io.read_u2be()
            self.sw_ana_cdh_temp = self._io.read_u2be()
            self.sw_ana_cdh_3p3_ref = self._io.read_u2be()
            self.sw_ana_sa1_v = self._io.read_u2be()
            self.sw_ana_sa1_i = self._io.read_u2be()
            self.sw_ana_sa2_v = self._io.read_u2be()
            self.sw_ana_sa2_i = self._io.read_u2be()
            self.sw_ana_bat1_v = self._io.read_u2be()
            self.sw_ana_eps_temp = self._io.read_u2be()
            self.sw_ana_eps_3p3_ref = self._io.read_u2be()
            self.sw_ana_eps_bus_v = self._io.read_u2be()
            self.sw_ana_eps_bus_i = self._io.read_u2be()
            self.sw_ana_xact_v = self._io.read_u2be()
            self.sw_ana_xact_i = self._io.read_u2be()
            self.sw_ana_uhf_v = self._io.read_u2be()
            self.sw_ana_uhf_i = self._io.read_u2be()
            self.sw_ana_sband_v = self._io.read_u2be()
            self.sw_ana_sband_i = self._io.read_u2be()
            self.sw_ana_axis1_volt = self._io.read_u2be()
            self.sw_ana_axis1_curr = self._io.read_u2be()
            self.sw_ana_axis2_volt = self._io.read_u2be()
            self.sw_ana_axis2_curr = self._io.read_u2be()
            self.sw_ana_axis3_volt = self._io.read_u2be()
            self.sw_ana_axis3_curr = self._io.read_u2be()
            self.sw_ana_afire_volt = self._io.read_u2be()
            self.sw_ana_afire_curr = self._io.read_u2be()
            self.sw_ana_ifb_therm1 = self._io.read_u2be()
            self.sw_adcs_alive = self._io.read_u1()
            self.sw_adcs_eclipse = self._io.read_u1()
            self.sw_adcs_att_valid = self._io.read_bits_int_be(1) != 0
            self.sw_adcs_ref_valid = self._io.read_bits_int_be(1) != 0
            self.sw_adcs_time_valid = self._io.read_bits_int_be(1) != 0
            self.sw_adcs_mode = self._io.read_bits_int_be(1) != 0
            self.sw_adcs_recommend_sun_point = self._io.read_bits_int_be(1) != 0
            self.sw_adcs_sun_point_state = self._io.read_bits_int_be(3)
            self._io.align_to_byte()
            self.reusable_spare_17 = self._io.read_u1()
            self.sw_adcs_analogs_voltage_2p5 = self._io.read_u1()
            self.sw_adcs_analogs_voltage_1p8 = self._io.read_u1()
            self.sw_adcs_analogs_voltage_1p0 = self._io.read_u1()
            self.sw_adcs_analogs_det_temp = self._io.read_s1()
            self.sw_adcs_analogs_motor1_temp = self._io.read_s2be()
            self.sw_adcs_analogs_motor2_temp = self._io.read_s2be()
            self.sw_adcs_analogs_motor3_temp = self._io.read_s2be()
            self.sw_adcs_analogs_digital_bus_v = self._io.read_s2be()
            self.sw_adcs_cmd_acpt = self._io.read_u1()
            self.sw_adcs_cmd_fail = self._io.read_u1()
            self.sw_adcs_sun_vec1 = self._io.read_s2be()
            self.sw_adcs_sun_vec2 = self._io.read_s2be()
            self.sw_adcs_sun_vec3 = self._io.read_s2be()
            self.sw_adcs_wheel_sp1 = self._io.read_s2be()
            self.sw_adcs_wheel_sp2 = self._io.read_s2be()
            self.sw_adcs_wheel_sp3 = self._io.read_s2be()
            self.sw_adcs_body_rt1 = self._io.read_s4be()
            self.sw_adcs_body_rt2 = self._io.read_s4be()
            self.sw_adcs_body_rt3 = self._io.read_s4be()
            self.sw_adcs_body_quat1 = self._io.read_s4be()
            self.sw_adcs_body_quat2 = self._io.read_s4be()
            self.sw_adcs_body_quat3 = self._io.read_s4be()
            self.sw_adcs_body_quat4 = self._io.read_s4be()
            self.des_met_time_seconds = self._io.read_u4be()
            self.sw_im_id = self._io.read_u1()
            self.payload_alive_axis1 = self._io.read_u1()
            self.payload_alive_axis2 = self._io.read_u1()
            self.payload_alive_axis3 = self._io.read_u1()
            self.payload_alive_afire = self._io.read_u1()
            self.reusable_spare_18 = self._io.read_u1()
            self.reusable_spare_19 = self._io.read_u1()
            self.reusable_spare_20 = self._io.read_u1()
            self.checksum = self._io.read_u4be()


    class SecondaryHeaderT(KaitaiStruct):
        """The Secondary Header is a feature of the Space Packet which allows
        additional types of information that may be useful to the user
        application (e.g., a time code) to be included.
        See: 4.1.3.2 in CCSDS 133.0-B-1
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.time_stamp_seconds = self._io.read_u4be()
            self.sub_seconds = self._io.read_u2be()


    class PacketPrimaryHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_version = self._io.read_bits_int_be(3)
            self.packet_type = self._io.read_bits_int_be(1) != 0
            self.secondary_header_flag = self._io.read_bits_int_be(1) != 0
            self.is_stored_data = self._io.read_bits_int_be(1) != 0
            self.application_process_id = self._io.read_bits_int_be(10)
            self.grouping_flag = self._io.read_bits_int_be(2)
            self.sequence_count = self._io.read_bits_int_be(14)
            self._io.align_to_byte()
            self.packet_length = self._io.read_u2be()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Aepex.Callsign(_io__raw_callsign_ror, self, self._root)


    class CcsdsSpacePacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_packet_primary_header = self._io.read_bytes(6)
            _io__raw_packet_primary_header = KaitaiStream(BytesIO(self._raw_packet_primary_header))
            self.packet_primary_header = Aepex.PacketPrimaryHeaderT(_io__raw_packet_primary_header, self, self._root)
            self.data_section = Aepex.DataSectionT(self._io, self, self._root)


    class AepexSwStat(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sw_stat_t = Aepex.SwStatT(self._io, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_space_packet = Aepex.CcsdsSpacePacketT(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Foresail1p(KaitaiStruct):
    """:field version_and_identity_len: skylink.version_and_identity_len
    :field identity: skylink.identity
    :field flags: skylink.header.flags
    :field frame_sequence: skylink.header.frame_sequence
    :field extension_length: skylink.header.extension_length
    :field vc: skylink.header.vc
    :field is_arq_on: skylink.header.is_arq_on
    :field is_authenticated: skylink.header.is_authenticated
    :field is_crced: skylink.header.is_crced
    :field sequence_control: skylink.header.sequence_control
    :field version: skylink.version
    :field len_identity: skylink.len_identity
    :field packet_id: pus.header.packet_id
    :field sequence: pus.header.sequence
    :field length: pus.header.length
    :field secondary_header: pus.header.secondary_header
    :field service_type: pus.header.service_type
    :field service_subtype: pus.header.service_subtype
    
    :field timestamp: pus.obc_housekeeping.timestamp.timestamp
    :field side: pus.obc_housekeeping.side
    :field fdir_mode: pus.obc_housekeeping.fdir_mode
    :field sys_watchdog_counter: pus.obc_housekeeping.sys_watchdog_counter
    :field scheduler: pus.obc_housekeeping.scheduler
    :field software_revision: pus.obc_housekeeping.software_revision
    :field uptime: pus.obc_housekeeping.uptime
    :field heap_free: pus.obc_housekeeping.heap_free
    :field cpu_load: pus.obc_housekeeping.cpu_load
    :field fs_free_space: pus.obc_housekeeping.fs_free_space
    :field arbiter_uptime: pus.obc_housekeeping.arbiter_uptime
    :field arbiter_age: pus.obc_housekeeping.arbiter_age
    :field arbiter_bootcount: pus.obc_housekeeping.arbiter_bootcount
    :field arbiter_temperature: pus.obc_housekeeping.arbiter_temperature
    :field side_a_bootcount: pus.obc_housekeeping.side_a_bootcount
    :field side_a_heartbeat: pus.obc_housekeeping.side_a_heartbeat
    :field side_a_fail_counter: pus.obc_housekeeping.side_a_fail_counter
    :field side_a_fail_reason: pus.obc_housekeeping.side_a_fail_reason
    :field side_b_bootcount: pus.obc_housekeeping.side_b_bootcount
    :field side_b_heartbeat: pus.obc_housekeeping.side_b_heartbeat
    :field side_b_fail_counter: pus.obc_housekeeping.side_b_fail_counter
    :field side_b_fail_reason: pus.obc_housekeeping.side_b_fail_reason
    :field arbiter_log_1: pus.obc_housekeeping.arbiter_log_1
    :field arbiter_log_2: pus.obc_housekeeping.arbiter_log_2
    :field arbiter_log_3: pus.obc_housekeeping.arbiter_log_3
    :field arbiter_log_4: pus.obc_housekeeping.arbiter_log_4
    
    :field eps_housekeeping_timestamp: pus.eps_housekeeping.timestamp.timestamp
    :field pcdu_uptime: pus.eps_housekeeping.pcdu_uptime
    :field pcdu_boot_count: pus.eps_housekeeping.pcdu_boot_count
    :field bb_boot_count: pus.eps_housekeeping.bb_boot_count
    :field apr_boot_count: pus.eps_housekeeping.apr_boot_count
    :field pdm_expected: pus.eps_housekeeping.pdm_expected
    :field pdm_faulted: pus.eps_housekeeping.pdm_faulted
    :field padding_byte: pus.eps_housekeeping.padding_byte
    :field eps_subsystem_state: pus.eps_housekeeping.eps_subsystem_state
    :field heater_pwm_on_time: pus.eps_housekeeping.heater_pwm_on_time
    :field current_apr_x: pus.eps_housekeeping.current_apr_x
    :field current_apr_y: pus.eps_housekeeping.current_apr_y
    :field voltage_apr_x: pus.eps_housekeeping.voltage_apr_x
    :field voltage_apr_y: pus.eps_housekeeping.voltage_apr_y
    :field current_apr_x_at_mpp: pus.eps_housekeeping.current_apr_x_at_mpp
    :field voltage_apr_x_at_mpp: pus.eps_housekeeping.voltage_apr_x_at_mpp
    :field current_apr_y_at_mpp: pus.eps_housekeeping.current_apr_y_at_mpp
    :field voltage_apr_y_at_mpp: pus.eps_housekeeping.voltage_apr_y_at_mpp
    :field voltage_battery: pus.eps_housekeeping.voltage_battery
    :field voltage_battery_lower: pus.eps_housekeeping.voltage_battery_lower
    :field voltage_payloads: pus.eps_housekeeping.voltage_payloads
    :field voltage_obc_adcs: pus.eps_housekeeping.voltage_obc_adcs
    :field voltage_uhf: pus.eps_housekeeping.voltage_uhf
    :field temperature_mcu_pcdu: pus.eps_housekeeping.temperature_mcu_pcdu
    :field temperature_mcu_bb: pus.eps_housekeeping.temperature_mcu_bb
    :field temperature_mcu_apr: pus.eps_housekeeping.temperature_mcu_apr
    :field temperature_battery: pus.eps_housekeeping.temperature_battery
    :field temperature_sp_x_minus: pus.eps_housekeeping.temperature_sp_x_minus
    :field temperature_sp_x_plus: pus.eps_housekeeping.temperature_sp_x_plus
    :field temperature_sp_y_minus: pus.eps_housekeeping.temperature_sp_y_minus
    :field temperature_sp_y_plus: pus.eps_housekeeping.temperature_sp_y_plus
    :field current_battery: pus.eps_housekeeping.current_battery
    :field current_battery_min: pus.eps_housekeeping.current_battery_min
    :field current_battery_max: pus.eps_housekeeping.current_battery_max
    :field current_pate_batt: pus.eps_housekeeping.current_pate_batt
    :field current_pb_batt: pus.eps_housekeeping.current_pb_batt
    :field current_pb_3v6: pus.eps_housekeeping.current_pb_3v6
    :field current_cam_3v6: pus.eps_housekeeping.current_cam_3v6
    :field current_mag_3v6: pus.eps_housekeeping.current_mag_3v6
    :field current_obc_3v6: pus.eps_housekeeping.current_obc_3v6
    :field current_uhf_3v6: pus.eps_housekeeping.current_uhf_3v6
    :field current_adcs_3v6: pus.eps_housekeeping.current_adcs_3v6
    :field current_min_pate: pus.eps_housekeeping.current_min_pate
    :field current_min_pb_batt: pus.eps_housekeeping.current_min_pb_batt
    :field current_min_pb_3v6: pus.eps_housekeeping.current_min_pb_3v6
    :field current_min_cam_3v6: pus.eps_housekeeping.current_min_cam_3v6
    :field current_min_mag_3v6: pus.eps_housekeeping.current_min_mag_3v6
    :field current_min_obc_3v6: pus.eps_housekeeping.current_min_obc_3v6
    :field current_min_uhf_3v6: pus.eps_housekeeping.current_min_uhf_3v6
    :field current_min_adcs_3v6: pus.eps_housekeeping.current_min_adcs_3v6
    :field current_max_pate: pus.eps_housekeeping.current_max_pate
    :field current_max_pb_batt: pus.eps_housekeeping.current_max_pb_batt
    :field current_max_pb_3v6: pus.eps_housekeeping.current_max_pb_3v6
    :field current_max_cam_3v6: pus.eps_housekeeping.current_max_cam_3v6
    :field current_max_mag_3v6: pus.eps_housekeeping.current_max_mag_3v6
    :field current_max_obc_3v6: pus.eps_housekeeping.current_max_obc_3v6
    :field current_max_uhf_3v6: pus.eps_housekeeping.current_max_uhf_3v6
    :field current_max_adcs_3v6: pus.eps_housekeeping.current_max_adcs_3v6
    
    :field uhf_housekeeping_timestamp: pus.uhf_housekeeping.timestamp.timestamp
    :field uhf_housekeeping_uptime: pus.uhf_housekeeping.uptime
    :field bootcount: pus.uhf_housekeeping.bootcount
    :field fdir_counter: pus.uhf_housekeeping.fdir_counter
    :field mcu_wd_reset_count: pus.uhf_housekeeping.mcu_wd_reset_count
    :field mbe_count: pus.uhf_housekeeping.mbe_count
    :field bus_sync_errors: pus.uhf_housekeeping.bus_sync_errors
    :field bus_len_errors: pus.uhf_housekeeping.bus_len_errors
    :field bus_crc_errors: pus.uhf_housekeeping.bus_crc_errors
    :field bus_receive_timeouts: pus.uhf_housekeeping.bus_receive_timeouts
    :field total_tx_frames: pus.uhf_housekeeping.total_tx_frames
    :field total_rx_frames: pus.uhf_housekeeping.total_rx_frames
    :field total_ham_tx_frames: pus.uhf_housekeeping.total_ham_tx_frames
    :field total_ham_rx_frames: pus.uhf_housekeeping.total_ham_rx_frames
    :field hardware_side_a_bootcount: pus.uhf_housekeeping.hardware_side_a_bootcount
    :field hardware_side_b_bootcount: pus.uhf_housekeeping.hardware_side_b_bootcount
    :field uhf_housekeeping_side: pus.uhf_housekeeping.side
    :field symbol_rate_rx: pus.uhf_housekeeping.symbol_rate_rx
    :field symbol_rate_tx: pus.uhf_housekeeping.symbol_rate_tx
    :field my_window_length: pus.uhf_housekeeping.my_window_length
    :field peer_window_length: pus.uhf_housekeeping.peer_window_length
    :field mcu_temperature: pus.uhf_housekeeping.mcu_temperature
    :field pa_temperature: pus.uhf_housekeeping.pa_temperature
    :field background_rssi: pus.uhf_housekeeping.background_rssi
    :field background_max_rssi: pus.uhf_housekeeping.background_max_rssi
    :field last_rssi: pus.uhf_housekeeping.last_rssi
    :field last_freq_offset: pus.uhf_housekeeping.last_freq_offset
    
    :field adcs_housekeeping_timestamp: pus.adcs_housekeeping.timestamp.timestamp
    :field determination_mode: pus.adcs_housekeeping.determination_mode
    :field control_mode: pus.adcs_housekeeping.control_mode
    :field mjd: pus.adcs_housekeeping.mjd
    :field position_x: pus.adcs_housekeeping.position_x
    :field position_y: pus.adcs_housekeeping.position_y
    :field position_z: pus.adcs_housekeeping.position_z
    :field velocity_x: pus.adcs_housekeeping.velocity_x
    :field velocity_y: pus.adcs_housekeeping.velocity_y
    :field velocity_z: pus.adcs_housekeeping.velocity_z
    :field low: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.low
    :field high: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.high
    :field combined_u5: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.combined_u5
    :field q1: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q1
    :field q2: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q2
    :field q3: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q3
    :field q4_metadata: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q4_metadata
    :field q4_index: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q4_index
    :field q4_sign: pus.adcs_housekeeping.estimated_attitude_compressed_quaternion.q4_sign
    :field estimated_angular_rate_x: pus.adcs_housekeeping.estimated_angular_rate_x
    :field estimated_angular_rate_y: pus.adcs_housekeeping.estimated_angular_rate_y
    :field estimated_angular_rate_z: pus.adcs_housekeeping.estimated_angular_rate_z
    :field estimated_magnetometer_bias_x: pus.adcs_housekeeping.estimated_magnetometer_bias_x
    :field estimated_magnetometer_bias_y: pus.adcs_housekeeping.estimated_magnetometer_bias_y
    :field estimated_magnetometer_bias_z: pus.adcs_housekeeping.estimated_magnetometer_bias_z
    :field estimated_gyro_bias_x: pus.adcs_housekeeping.estimated_gyro_bias_x
    :field estimated_gyro_bias_y: pus.adcs_housekeeping.estimated_gyro_bias_y
    :field estimated_gyro_bias_z: pus.adcs_housekeeping.estimated_gyro_bias_z
    :field attitude_variance: pus.adcs_housekeeping.attitude_variance
    :field angular_velocity_variance: pus.adcs_housekeeping.angular_velocity_variance
    :field mag_bias_variance: pus.adcs_housekeeping.mag_bias_variance
    :field gyro_bias_variance: pus.adcs_housekeeping.gyro_bias_variance
    :field magnetometer_measurement_x: pus.adcs_housekeeping.magnetometer_measurement_x
    :field magnetometer_measurement_y: pus.adcs_housekeeping.magnetometer_measurement_y
    :field magnetometer_measurement_z: pus.adcs_housekeeping.magnetometer_measurement_z
    :field gyroscope_measurement_x: pus.adcs_housekeeping.gyroscope_measurement_x
    :field gyroscope_measurement_y: pus.adcs_housekeeping.gyroscope_measurement_y
    :field gyroscope_measurement_z: pus.adcs_housekeeping.gyroscope_measurement_z
    :field sun_vector_measurement_x: pus.adcs_housekeeping.sun_vector_measurement_x
    :field sun_vector_measurement_y: pus.adcs_housekeeping.sun_vector_measurement_y
    :field sun_vector_measurement_z: pus.adcs_housekeeping.sun_vector_measurement_z
    
    :field event_timestamp: pus.event.timestamp.timestamp
    :field rid: pus.event.rid
    :field info: pus.event.info.b64encstring.info_data_b64_encoded
    
    :field callsign: repeater.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: repeater.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: repeater.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: repeater.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: repeater.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: repeater.ax25_header.src_ssid_raw.ssid
    :field ctl: repeater.ax25_header.ctl
    :field pid: repeater.ax25_header.pid
    :field payload: repeater.payload
    :field fcs: repeater.fcs
    
    .. seealso::
       Source - https://foresail.github.io/docs/FS1p_Space_Ground_Interface_Control_Sheet.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.skylink = Foresail1p.SkylinkFrame(self._io, self, self._root)
        if  ((self.skylink.header.vc == 0) or (self.skylink.header.vc == 1)) :
            self._raw_pus = self._io.read_bytes(((self._io.size() - self._io.pos()) - (4 * self.skylink.header.is_authenticated)))
            _io__raw_pus = KaitaiStream(BytesIO(self._raw_pus))
            self.pus = Foresail1p.ForesailPusFrame(_io__raw_pus, self, self._root)

        if self.skylink.header.vc == 3:
            self._raw_repeater = self._io.read_bytes(((self._io.size() - self._io.pos()) - (4 * self.skylink.header.is_crced)))
            _io__raw_repeater = KaitaiStream(BytesIO(self._raw_repeater))
            self.repeater = Foresail1p.Ax25Frame(_io__raw_repeater, self, self._root)

        if self.skylink.header.is_authenticated == 1:
            self.auth = self._io.read_bytes(4)

        if self.skylink.header.is_crced == 1:
            self.frame_crc = self._io.read_bytes(4)


    class Ax25Frame(KaitaiStruct):
        """
        .. seealso::
           Source - https://www.tapr.org/pub_ax25.html
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Foresail1p.Ax25Header(self._io, self, self._root)
            self.payload = (self._io.read_bytes(((self._io.size() - self._io.pos()) - 2))).decode(u"ASCII")
            self.fcs = self._io.read_u2be()


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Foresail1p.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Foresail1p.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Foresail1p.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Foresail1p.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.digipeaters = []
                i = 0
                while True:
                    _ = Foresail1p.Digipeater(self._io, self, self._root)
                    self.digipeaters.append(_)
                    if (_.ssid_raw.ssid_mask & 1) != 0:
                        break
                    i += 1

            self.ctl = self._io.read_u1()
            if not self.ctl == 3:
                raise kaitaistruct.ValidationNotEqualError(3, self.ctl, self._io, u"/types/ax25_header/seq/5")
            self.pid = self._io.read_u1()
            if not self.pid == 240:
                raise kaitaistruct.ValidationNotEqualError(240, self.pid, self._io, u"/types/ax25_header/seq/6")


    class SkyStaticHeader(KaitaiStruct):
        """Skylink static header."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.flags = self._io.read_u1()
            self.frame_sequence = self._io.read_u2be()
            self.extension_length = self._io.read_u1()

        @property
        def is_arq_on(self):
            if hasattr(self, '_m_is_arq_on'):
                return self._m_is_arq_on

            self._m_is_arq_on = ((self.flags & 4) >> 2)
            return getattr(self, '_m_is_arq_on', None)

        @property
        def is_crced(self):
            if hasattr(self, '_m_is_crced'):
                return self._m_is_crced

            self._m_is_crced = ((self.flags & 16) >> 4)
            return getattr(self, '_m_is_crced', None)

        @property
        def sequence_control(self):
            if hasattr(self, '_m_sequence_control'):
                return self._m_sequence_control

            self._m_sequence_control = ((self.flags & 96) >> 5)
            return getattr(self, '_m_sequence_control', None)

        @property
        def is_authenticated(self):
            if hasattr(self, '_m_is_authenticated'):
                return self._m_is_authenticated

            self._m_is_authenticated = ((self.flags & 8) >> 3)
            return getattr(self, '_m_is_authenticated', None)

        @property
        def vc(self):
            if hasattr(self, '_m_vc'):
                return self._m_vc

            self._m_vc = (self.flags & 3)
            return getattr(self, '_m_vc', None)


    class PusHeader(KaitaiStruct):
        """CCSDS PUS header."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_id = self._io.read_u2be()
            self.sequence = self._io.read_u2be()
            self.length = self._io.read_u2be()
            self.secondary_header = self._io.read_u1()
            self.service_type = self._io.read_u1()
            self.service_subtype = self._io.read_u1()


    class AdcsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1p.UnixTimestamp(self._io, self, self._root)
            self.determination_mode = self._io.read_u1()
            self.control_mode = self._io.read_u1()
            self.mjd = self._io.read_f4le()
            self.position_x = self._io.read_f4le()
            self.position_y = self._io.read_f4le()
            self.position_z = self._io.read_f4le()
            self.velocity_x = self._io.read_f4le()
            self.velocity_y = self._io.read_f4le()
            self.velocity_z = self._io.read_f4le()
            self.estimated_attitude_compressed_quaternion = Foresail1p.CompressedQuaternion(self._io, self, self._root)
            self.estimated_angular_rate_x = self._io.read_u2le()
            self.estimated_angular_rate_y = self._io.read_u2le()
            self.estimated_angular_rate_z = self._io.read_u2le()
            self.estimated_magnetometer_bias_x = self._io.read_u2le()
            self.estimated_magnetometer_bias_y = self._io.read_u2le()
            self.estimated_magnetometer_bias_z = self._io.read_u2le()
            self.estimated_gyro_bias_x = self._io.read_u2le()
            self.estimated_gyro_bias_y = self._io.read_u2le()
            self.estimated_gyro_bias_z = self._io.read_u2le()
            self.attitude_variance = self._io.read_u2le()
            self.angular_velocity_variance = self._io.read_u2le()
            self.mag_bias_variance = self._io.read_u2le()
            self.gyro_bias_variance = self._io.read_u2le()
            self.magnetometer_measurement_x = self._io.read_s2le()
            self.magnetometer_measurement_y = self._io.read_s2le()
            self.magnetometer_measurement_z = self._io.read_s2le()
            self.gyroscope_measurement_x = self._io.read_s2le()
            self.gyroscope_measurement_y = self._io.read_s2le()
            self.gyroscope_measurement_z = self._io.read_s2le()
            self.sun_vector_measurement_x = self._io.read_s2le()
            self.sun_vector_measurement_y = self._io.read_s2le()
            self.sun_vector_measurement_z = self._io.read_s2le()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class UhfHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1p.UnixTimestamp(self._io, self, self._root)
            self.uptime = self._io.read_u4le()
            self.bootcount = self._io.read_u2le()
            self.fdir_counter = self._io.read_u1()
            self.mcu_wd_reset_count = self._io.read_u1()
            self.mbe_count = self._io.read_u1()
            self.bus_sync_errors = self._io.read_u1()
            self.bus_len_errors = self._io.read_u1()
            self.bus_crc_errors = self._io.read_u1()
            self.bus_receive_timeouts = self._io.read_u1()
            self.total_tx_frames = self._io.read_u4le()
            self.total_rx_frames = self._io.read_u4le()
            self.total_ham_tx_frames = self._io.read_u4le()
            self.total_ham_rx_frames = self._io.read_u4le()
            self.hardware_side_a_bootcount = self._io.read_u1()
            self.hardware_side_b_bootcount = self._io.read_u1()
            self.side = self._io.read_u1()
            self.symbol_rate_rx = self._io.read_u1()
            self.symbol_rate_tx = self._io.read_u1()
            self.my_window_length = self._io.read_u4le()
            self.peer_window_length = self._io.read_u4le()
            self.mcu_temperature = self._io.read_s2le()
            self.pa_temperature = self._io.read_s2le()
            self.background_rssi = self._io.read_s2le()
            self.background_max_rssi = self._io.read_s2le()
            self.last_rssi = self._io.read_s2le()
            self.last_freq_offset = self._io.read_s2le()


    class UnixTimestamp(KaitaiStruct):
        """Unix timestamp."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = self._io.read_u4be()


    class Event(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1p.UnixTimestamp(self._io, self, self._root)
            self.rid = self._io.read_u2be()
            self._raw_info = self._io.read_bytes_full()
            _io__raw_info = KaitaiStream(BytesIO(self._raw_info))
            self.info = Foresail1p.Event.Base64(_io__raw_info, self, self._root)

        class Base64(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_b64encstring = self._io.read_bytes_full()
                _process = satnogsdecoders.process.B64encode()
                self._raw_b64encstring = _process.decode(self._raw__raw_b64encstring)
                _io__raw_b64encstring = KaitaiStream(BytesIO(self._raw_b64encstring))
                self.b64encstring = Foresail1p.Event.Base64string(_io__raw_b64encstring, self, self._root)


        class Base64string(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.info_data_b64_encoded = (self._io.read_bytes_full()).decode(u"UTF-8")



    class Digipeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_raw = Foresail1p.CallsignRaw(self._io, self, self._root)
            self.ssid_raw = Foresail1p.SsidMask(self._io, self, self._root)

        @property
        def has_been_repeated(self):
            if hasattr(self, '_m_has_been_repeated'):
                return self._m_has_been_repeated

            self._m_has_been_repeated = (self.ssid_raw.ssid_mask & 128) != 0
            return getattr(self, '_m_has_been_repeated', None)


    class EpsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1p.UnixTimestamp(self._io, self, self._root)
            self.pcdu_uptime = self._io.read_u4le()
            self.pcdu_boot_count = self._io.read_u1()
            self.bb_boot_count = self._io.read_u1()
            self.apr_boot_count = self._io.read_u1()
            self.pdm_expected = self._io.read_u1()
            self.pdm_faulted = self._io.read_u1()
            self.padding_byte = self._io.read_u1()
            self.eps_subsystem_state = self._io.read_u2le()
            self.heater_pwm_on_time = self._io.read_u2le()
            self.current_apr_x = self._io.read_s2le()
            self.current_apr_y = self._io.read_s2le()
            self.voltage_apr_x = self._io.read_s2le()
            self.voltage_apr_y = self._io.read_s2le()
            self.current_apr_x_at_mpp = self._io.read_s2le()
            self.voltage_apr_x_at_mpp = self._io.read_s2le()
            self.current_apr_y_at_mpp = self._io.read_s2le()
            self.voltage_apr_y_at_mpp = self._io.read_s2le()
            self.voltage_battery = self._io.read_s2le()
            self.voltage_battery_lower = self._io.read_s2le()
            self.voltage_payloads = self._io.read_s2le()
            self.voltage_obc_adcs = self._io.read_s2le()
            self.voltage_uhf = self._io.read_s2le()
            self.temperature_mcu_pcdu = self._io.read_s2le()
            self.temperature_mcu_bb = self._io.read_s2le()
            self.temperature_mcu_apr = self._io.read_s2le()
            self.temperature_battery = self._io.read_s2le()
            self.temperature_sp_x_minus = self._io.read_s2le()
            self.temperature_sp_x_plus = self._io.read_s2le()
            self.temperature_sp_y_minus = self._io.read_s2le()
            self.temperature_sp_y_plus = self._io.read_s2le()
            self.current_battery = self._io.read_s2le()
            self.current_battery_min = self._io.read_s2le()
            self.current_battery_max = self._io.read_s2le()
            self.current_pate_batt = self._io.read_s2le()
            self.current_pb_batt = self._io.read_s2le()
            self.current_pb_3v6 = self._io.read_s2le()
            self.current_cam_3v6 = self._io.read_s2le()
            self.current_mag_3v6 = self._io.read_s2le()
            self.current_obc_3v6 = self._io.read_s2le()
            self.current_uhf_3v6 = self._io.read_s2le()
            self.current_adcs_3v6 = self._io.read_s2le()
            self.current_min_pate = self._io.read_s2le()
            self.current_min_pb_batt = self._io.read_s2le()
            self.current_min_pb_3v6 = self._io.read_s2le()
            self.current_min_cam_3v6 = self._io.read_s2le()
            self.current_min_mag_3v6 = self._io.read_s2le()
            self.current_min_obc_3v6 = self._io.read_s2le()
            self.current_min_uhf_3v6 = self._io.read_s2le()
            self.current_min_adcs_3v6 = self._io.read_s2le()
            self.current_max_pate = self._io.read_s2le()
            self.current_max_pb_batt = self._io.read_s2le()
            self.current_max_pb_3v6 = self._io.read_s2le()
            self.current_max_cam_3v6 = self._io.read_s2le()
            self.current_max_mag_3v6 = self._io.read_s2le()
            self.current_max_obc_3v6 = self._io.read_s2le()
            self.current_max_uhf_3v6 = self._io.read_s2le()
            self.current_max_adcs_3v6 = self._io.read_s2le()


    class ForesailPusFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = Foresail1p.PusHeader(self._io, self, self._root)
            if  ((self.header.service_type == 3) and (self.header.service_subtype == 2)) :
                self.obc_housekeeping = Foresail1p.ObcHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 3)) :
                self.eps_housekeeping = Foresail1p.EpsHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 4)) :
                self.uhf_housekeeping = Foresail1p.UhfHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 3) and (self.header.service_subtype == 5)) :
                self.adcs_housekeeping = Foresail1p.AdcsHousekeeping(self._io, self, self._root)

            if  ((self.header.service_type == 4) and ( ((self.header.service_subtype == 1) or (self.header.service_subtype == 2) or (self.header.service_subtype == 3) or (self.header.service_subtype == 4)) )) :
                self.event = Foresail1p.Event(self._io, self, self._root)



    class Pdms(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.val = self._io.read_u1()

        @property
        def mag_3v6(self):
            if hasattr(self, '_m_mag_3v6'):
                return self._m_mag_3v6

            self._m_mag_3v6 = ((self.val & 16) >> 4)
            return getattr(self, '_m_mag_3v6', None)

        @property
        def uhf_3v6(self):
            if hasattr(self, '_m_uhf_3v6'):
                return self._m_uhf_3v6

            self._m_uhf_3v6 = ((self.val & 64) >> 6)
            return getattr(self, '_m_uhf_3v6', None)

        @property
        def pate_batt(self):
            if hasattr(self, '_m_pate_batt'):
                return self._m_pate_batt

            self._m_pate_batt = ((self.val & 1) >> 0)
            return getattr(self, '_m_pate_batt', None)

        @property
        def obc_3v6(self):
            if hasattr(self, '_m_obc_3v6'):
                return self._m_obc_3v6

            self._m_obc_3v6 = ((self.val & 32) >> 5)
            return getattr(self, '_m_obc_3v6', None)

        @property
        def cam_3v6(self):
            if hasattr(self, '_m_cam_3v6'):
                return self._m_cam_3v6

            self._m_cam_3v6 = ((self.val & 8) >> 3)
            return getattr(self, '_m_cam_3v6', None)

        @property
        def pb_3v6(self):
            if hasattr(self, '_m_pb_3v6'):
                return self._m_pb_3v6

            self._m_pb_3v6 = ((self.val & 4) >> 2)
            return getattr(self, '_m_pb_3v6', None)

        @property
        def adcs_3v6(self):
            if hasattr(self, '_m_adcs_3v6'):
                return self._m_adcs_3v6

            self._m_adcs_3v6 = ((self.val & 128) >> 7)
            return getattr(self, '_m_adcs_3v6', None)

        @property
        def pb_batt(self):
            if hasattr(self, '_m_pb_batt'):
                return self._m_pb_batt

            self._m_pb_batt = ((self.val & 2) >> 1)
            return getattr(self, '_m_pb_batt', None)


    class SkylinkFrame(KaitaiStruct):
        """
        .. seealso::
           Skylink Protocol Specification.pdf
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.version_and_identity_len = self._io.read_u1()
            self.identity = (self._io.read_bytes(self.len_identity)).decode(u"ASCII")
            if not self.identity == u"OH2F1S":
                raise kaitaistruct.ValidationNotEqualError(u"OH2F1S", self.identity, self._io, u"/types/skylink_frame/seq/1")
            self.header = Foresail1p.SkyStaticHeader(self._io, self, self._root)
            self.extensions = self._io.read_bytes(self.header.extension_length)

        @property
        def version(self):
            if hasattr(self, '_m_version'):
                return self._m_version

            self._m_version = ((self.version_and_identity_len & 248) >> 3)
            return getattr(self, '_m_version', None)

        @property
        def len_identity(self):
            if hasattr(self, '_m_len_identity'):
                return self._m_len_identity

            self._m_len_identity = (self.version_and_identity_len & 7)
            return getattr(self, '_m_len_identity', None)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class CompressedQuaternion(KaitaiStruct):
        """Metadata for the last component:
        - 2 bits: Index of the largest component (0-2)
        - 1 bit: Sign of the largest component (0 = positive, 1 = negative)
        - 1 bit: Unused
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.low = self._io.read_u4le()
            self.high = self._io.read_u1()

        @property
        def q4_sign(self):
            if hasattr(self, '_m_q4_sign'):
                return self._m_q4_sign

            self._m_q4_sign = ((self.q4_metadata >> 1) & 1)
            return getattr(self, '_m_q4_sign', None)

        @property
        def q1(self):
            if hasattr(self, '_m_q1'):
                return self._m_q1

            self._m_q1 = ((self.combined_u5 >> 28) & 4095)
            return getattr(self, '_m_q1', None)

        @property
        def combined_u5(self):
            if hasattr(self, '_m_combined_u5'):
                return self._m_combined_u5

            self._m_combined_u5 = ((self.low << 8) | self.high)
            return getattr(self, '_m_combined_u5', None)

        @property
        def q4_metadata(self):
            if hasattr(self, '_m_q4_metadata'):
                return self._m_q4_metadata

            self._m_q4_metadata = (self.combined_u5 & 15)
            return getattr(self, '_m_q4_metadata', None)

        @property
        def q4_index(self):
            if hasattr(self, '_m_q4_index'):
                return self._m_q4_index

            self._m_q4_index = ((self.q4_metadata >> 2) & 3)
            return getattr(self, '_m_q4_index', None)

        @property
        def q2(self):
            if hasattr(self, '_m_q2'):
                return self._m_q2

            self._m_q2 = ((self.combined_u5 >> 16) & 4095)
            return getattr(self, '_m_q2', None)

        @property
        def q3(self):
            if hasattr(self, '_m_q3'):
                return self._m_q3

            self._m_q3 = ((self.combined_u5 >> 4) & 4095)
            return getattr(self, '_m_q3', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Foresail1p.Callsign(_io__raw_callsign_ror, self, self._root)


    class ObcHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = Foresail1p.UnixTimestamp(self._io, self, self._root)
            self.side = self._io.read_u1()
            self.fdir_mode = self._io.read_u1()
            self.sys_watchdog_counter = self._io.read_u2le()
            self.scheduler = self._io.read_u1()
            self.software_revision = self._io.read_u1()
            self.uptime = self._io.read_u4le()
            self.heap_free = self._io.read_u1()
            self.cpu_load = self._io.read_u1()
            self.fs_free_space = self._io.read_u2le()
            self.arbiter_uptime = self._io.read_u2le()
            self.arbiter_age = self._io.read_u2le()
            self.arbiter_bootcount = self._io.read_u2le()
            self.arbiter_temperature = self._io.read_s2le()
            self.side_a_bootcount = self._io.read_u1()
            self.side_a_heartbeat = self._io.read_u1()
            self.side_a_fail_counter = self._io.read_u1()
            self.side_a_fail_reason = self._io.read_u1()
            self.side_b_bootcount = self._io.read_u1()
            self.side_b_heartbeat = self._io.read_u1()
            self.side_b_fail_counter = self._io.read_u1()
            self.side_b_fail_reason = self._io.read_u1()
            self.arbiter_log_1 = self._io.read_u2le()
            self.arbiter_log_2 = self._io.read_u2le()
            self.arbiter_log_3 = self._io.read_u2le()
            self.arbiter_log_4 = self._io.read_u2le()






"""
Preprocessor used in kaitai struct to encode data into hexl
encoded data
"""
import binascii


class Unhexl(object):  # pylint: disable=too-few-public-methods
    """
    Hexl preprocessor class
    """

    def __init__(self):
        pass

    def decode(self, in_bindata):  # pylint: disable=no-self-use
        """
        unhexl method
        """
        bindata = bytearray(in_bindata)
        return bytearray(binascii.unhexlify(bindata))



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Tubin(KaitaiStruct):
    """:field msgtype: mf.msgtype
    :field block_length: mf.block_length
    :field target_address: mf.target_address
    :field target_sub_address: mf.target_sub_address
    :field ack_flag: mf.ack_flag
    :field baudrate_flag: mf.baudrate_flag
    :field callsign: mf.callsign
    :field callsign_crc: mf.callsign_crc
    :field scid: mf.tf.scid
    :field version: mf.tf.version
    :field counter: mf.tf.counter
    
    :field spcrc: mf.tf.sp.spcrc
    :field gs_quality_byte: mf.gs_quality_byte
    :field gs_error_marker: mf.gs_error_marker
    :field gs_tnc_temperature: mf.gs_tnc_temperature
    
    .. seealso::
       'https://www.tu.berlin/en/raumfahrttechnik/institute/amateur-radio'
       'https://www.static.tu.berlin/fileadmin/www/10002275/Amateur_Radio/TechnoSat_Telemetry_Format.ods'
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.mf = Tubin.MasterFrame(self._io, self, self._root)

    class SourcePacket(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.length_msb = self._io.read_u1()
            if self.length_msb != 255:
                self.length_lsb = self._io.read_u1()

            if self.length_msb == 255:
                self.padding = self._io.read_bytes_full()

            if self.length_msb != 255:
                self.vcid = self._io.read_bits_int_be(4)

            if self.length_msb != 255:
                self.rtfrac = self._io.read_bits_int_be(4)

            self._io.align_to_byte()
            if self.length_msb != 255:
                self.rtsec = self._io.read_u4be()

            if self.length_msb != 255:
                self.nodid = self._io.read_u1()

            if self.length_msb != 255:
                self.spid = self._io.read_u1()

            if self.length_msb != 255:
                _on = self.spid
                if _on == 131:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpTmAocsStateEstimationA(_io__raw_user_data, self, self._root)
                elif _on == 60:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpStdTmAocs(_io__raw_user_data, self, self._root)
                elif _on == 249:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpType249(_io__raw_user_data, self, self._root)
                elif _on == 21:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpType21(_io__raw_user_data, self, self._root)
                elif _on == 41:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpType41(_io__raw_user_data, self, self._root)
                elif _on == 16:
                    self._raw_user_data = self._io.read_bytes(self.splen)
                    _io__raw_user_data = KaitaiStream(BytesIO(self._raw_user_data))
                    self.user_data = Tubin.SpType16(_io__raw_user_data, self, self._root)
                else:
                    self.user_data = self._io.read_bytes(self.splen)

            if self.length_msb != 255:
                self.spcrc = self._io.read_u2be()


        @property
        def splen(self):
            if hasattr(self, '_m_splen'):
                return self._m_splen

            if self.length_msb != 255:
                self._m_splen = ((self.length_msb << 8) | (self.length_lsb if self.length_msb != 255 else 0))

            return getattr(self, '_m_splen', None)


    class SpType249(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes(3)


    class SpType16(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes(16)


    class SpType21(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes(28)


    class SpStdTmAocs(KaitaiStruct):
        """Type 60
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.nodeno = self._io.read_bits_int_be(1) != 0
            self.rst_en = self._io.read_bits_int_be(1) != 0
            self.botslt = self._io.read_bits_int_be(3)
            self.synpps = self._io.read_bits_int_be(1) != 0
            self.disutc = self._io.read_bits_int_be(1) != 0
            self.dulbsy = self._io.read_bits_int_be(1) != 0
            self.acs_mode = self._io.read_bits_int_be(5)
            self.mfs_received = self._io.read_bits_int_be(1) != 0
            self.sss_received = self._io.read_bits_int_be(1) != 0
            self.gyr_received = self._io.read_bits_int_be(1) != 0
            self.for_received = self._io.read_bits_int_be(1) != 0
            self.str_received = self._io.read_bits_int_be(1) != 0
            self.res_1_bit = self._io.read_bits_int_be(1) != 0
            self.mts_received = self._io.read_bits_int_be(1) != 0
            self.rw0_received = self._io.read_bits_int_be(1) != 0
            self.rw1_received = self._io.read_bits_int_be(1) != 0
            self.rw2_received = self._io.read_bits_int_be(1) != 0
            self.rw3_received = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.std_q_s = self._io.read_s2be()
            self.std_q_x = self._io.read_s2be()
            self.std_q_y = self._io.read_s2be()
            self.std_q_z = self._io.read_s2be()
            self.std_rate_x = self._io.read_s1()
            self.std_rate_y = self._io.read_s1()
            self.std_rate_z = self._io.read_s1()
            self.std_r_x = self._io.read_s1()
            self.std_r_y = self._io.read_s1()
            self.std_r_z = self._io.read_s1()


    class SpTmAocsStateEstimationA(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.esta_q_s = self._io.read_s4be()
            self.esta_q_x = self._io.read_s4be()
            self.esta_q_y = self._io.read_s4be()
            self.esta_q_z = self._io.read_s4be()
            self.esta_rate_x = self._io.read_s2be()
            self.esta_rate_y = self._io.read_s2be()
            self.esta_rate_z = self._io.read_s2be()
            self.esta_acc_x = self._io.read_s2be()
            self.esta_acc_y = self._io.read_s2be()
            self.esta_acc_z = self._io.read_s2be()
            self.esta_b_sat_x = self._io.read_s2be()
            self.esta_b_sat_y = self._io.read_s2be()
            self.esta_b_sat_z = self._io.read_s2be()
            self.esta_s_sat_x = self._io.read_s2be()
            self.esta_s_sat_y = self._io.read_s2be()
            self.esta_s_sat_z = self._io.read_s2be()
            self.esta_b_tod_x = self._io.read_s2be()
            self.esta_b_tod_y = self._io.read_s2be()
            self.esta_b_tod_z = self._io.read_s2be()
            self.esta_s_tod_x = self._io.read_s2be()
            self.esta_s_tod_y = self._io.read_s2be()
            self.esta_s_tod_z = self._io.read_s2be()
            self.esta_r_x = self._io.read_s2be()
            self.esta_r_y = self._io.read_s2be()
            self.esta_r_z = self._io.read_s2be()
            self.esta_v_x = self._io.read_s2be()
            self.esta_v_y = self._io.read_s2be()
            self.esta_v_z = self._io.read_s2be()
            self.esta_occultatio = self._io.read_u1()


    class MasterFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.msgtype = self._io.read_bits_int_be(3)
            self.block_length = self._io.read_bits_int_be(5)
            self.target_address = self._io.read_bits_int_be(4)
            self.target_sub_address = self._io.read_bits_int_be(2)
            self.ack_flag = self._io.read_bits_int_be(1) != 0
            self.baudrate_flag = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.callsign = (KaitaiStream.bytes_terminate(self._io.read_bytes(6), 0, False)).decode(u"ascii")
            if not self.callsign == u"DP0TBN":
                raise kaitaistruct.ValidationNotEqualError(u"DP0TBN", self.callsign, self._io, u"/types/master_frame/seq/6")
            self.callsign_crc = self._io.read_u2be()
            self._raw_tf = self._io.read_bytes(((self.block_length + 1) * 18))
            _io__raw_tf = KaitaiStream(BytesIO(self._raw_tf))
            self.tf = Tubin.TransferFrame(_io__raw_tf, self, self._root)
            self.gs_quality_byte = self._io.read_u1()
            self.gs_error_marker = self._io.read_u4be()
            self.gs_tnc_temperature = self._io.read_u1()


    class TransferFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.scid = self._io.read_bits_int_be(12)
            self.version = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.counter = self._io.read_u2be()
            self.sp = []
            i = 0
            while True:
                _ = Tubin.SourcePacket(self._io, self, self._root)
                self.sp.append(_)
                if  ((self._io.pos() >= self._io.size()) or (_.length_msb == 255)) :
                    break
                i += 1
            self.tfpad = self._io.read_bytes_full()


    class SpType41(KaitaiStruct):
        """Standard AOCS data
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes(7)



